unit ImageRipple;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TImageRipple = class(TImage)
  private
    Timer: TTimer;

    dest, source : TBitmap;
    dist, wave : PInteger;
    r_height, r_width, r_max,
    angle : integer;

    freq : double;

    procedure setFreq (aFreq : double);
  protected
    constructor create (owner : TComponent) ; override;
    procedure loaded; override;
    destructor destroy; override;
    procedure TimerTimer(Sender: TObject);
  public
  published
    property Frequency : double read freq write setFreq;
  end;

procedure Register;

implementation


constructor TImageRipple.create (owner : TComponent);
begin
  inherited;

  freq := 0.3;
end;


procedure TImageRipple.loaded;
var
  c, r, i : integer;
begin
  inherited;

  if csDesigning in componentState then
    exit;

  dest := picture.bitmap;
  source := TBitmap.create;
  source.assign (dest);
  source.canvas.draw (0, 0, dest);

  r_width := source.width div 2 + 1;
  r_height := source.height div 2 + 1;
  r_max := round (sqrt (r_height * r_height + r_width * r_width) + 2);

  getMem (dist, r_height * r_width * sizeOf (integer));
  getMem (wave, r_max * sizeOf (integer));

  for r := 0 to r_height - 1 do
    for c := 0 To r_width - 1 do
      PInteger (longint (dist)
                + (r * r_width + c) * sizeOf (integer))^ :=
        round (sqrt (r * r + c * c));
  { set up the square root values for speed }

  Timer := TTimer.create (nil);
  Timer.OnTimer := TimerTimer;
  Timer.Interval := 100;
  Timer.Enabled := true;

  TimerTimer (self);
end;


destructor TImageRipple.destroy;
begin
  source.free;
  freeMem (dist, r_height * r_width * sizeOf (integer));
  freeMem (wave, r_max * sizeOf (integer));

  Timer.free;

  inherited;
end;


procedure TImageRipple.TimerTimer(Sender: TObject);
var
  c, r, i, nc, waveVal, distVal : integer;
  destline, sourceLine : PByteArray;
  w : PInteger;
begin
  if (dest.PixelFormat <> pf8bit) then
    exit;
  { 256 color bitmaps only }

  angle := (angle + 1) mod r_max;
  { update the angle }

  w := wave;
  for i := 0 to r_max - 1 do
  begin
    w^ := round (15 * sin (freq * (i - angle)));
    inc (w);
  end;
  { update the wave }

  for r := 0 to dest.height - 1 do
    for c := 0 to dest.width - 1 do
    begin
      distVal := PInteger (longint (dist)
                           + (abs (r - r_height + 1) * r_width
                              + abs (c - r_width + 1)) * sizeOf (integer))^;
      { get the distance }

      waveVal := PInteger (longint (wave) + distVal * sizeOf (integer))^;
      { get the wave offset value for that distance }

      nc := abs (c - waveVal) {mod dest.width};
      { caluclate the offset }

      destLine := dest.scanLine [r];
      sourceLine := source.scanLine [r];

      destLine [c] := sourceLine [nc];
    end;
  { the critical loop }

  refresh;
end;


procedure TImageRipple.setFreq (aFreq : double);
begin
  freq := aFreq;
  if freq < 0.01 then
    freq := 0.01
  else if freq > 1 then
    freq := 1;
end;


procedure Register;
begin
  RegisterComponents('Samples', [TImageRipple]);
end;

end.
